/*
 * Name: uirep.c
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-02-13
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: uirep.c,v 1.10 2001/05/06 15:21:22 cs Exp $
 */

#include "stdheaders.h"
#include "uirep.h"
#include "process.h"
#include "blockpool.h"
#include "mainmodule.h"
#include "guimanager.h"
#include "memalloc.h"
#include "array.h"
#include "ipcserver.h"
#include "module.h"
#include "util.h"
#include "../../uiLib/maininterface.h"
#include "../../mainFramework/mainmodule.h"
#include "errcodes.h"
#include "ipc.h"
#include "cifsmod.h"
#include "unicode.h"

/* ------------------------------------------------------------------------- */

typedef struct uirepCtx{
	PROC_VARSET;
	btint			uid;
	btu8			scratch[128], host[128];
	uirepAuthData_t	*auth;
	array_t			*clients;
	btint			index;
	guiClass_t		guiClass;
	ipcSrvClient_t	*client;
}uirepCtx_t;

static blockpool_t *uirepPool = NULL;

static void	poolFreeBlock(void *block)
{
    poolReturnBlock(uirepPool, block);
}

/* ------------------------------------------------------------------------- */


#define	PANEL_OFFSET(entry)	(void *)&(((uirepAuthData_t *)NULL)->entry)

#define	ALLOW_NULL	0
#define	FLG_0		0

static guiEntry_t	passPhraseEntries[] = {
	{	MAIN_GUI_ENTRY_PASSWORD,	"PEM Pass Phrase",
		NULL,	"cifsDialogPassPhrase", MAIN_GUI_PERM_ANYBODY, 
		FLG_0, ALLOW_NULL,	PANEL_OFFSET(passwd)
	},
};

static guiEntry_t	loginPanelEntries[] = {
	{	MAIN_GUI_ENTRY_STRING,		"Remote Username",
		NULL,	"cifsDialogLoginRemoteUser", MAIN_GUI_PERM_ANYBODY, 
		FLG_0, ALLOW_NULL,	PANEL_OFFSET(username)
	},
	{	MAIN_GUI_ENTRY_PASSWORD,	"Password",
		NULL,	"cifsDialogLoginPassword", MAIN_GUI_PERM_ANYBODY, 
		MAIN_GUI_ENTRY_FLG_DEFAULT_RESPONDER,
		ALLOW_NULL,	PANEL_OFFSET(passwd)
	},
	{	MAIN_GUI_ENTRY_STRING,		"Domain",
		NULL,	"cifsDialogLoginDomain", MAIN_GUI_PERM_ANYBODY, 
		FLG_0, ALLOW_NULL,	PANEL_OFFSET(domain)
	},
	{	MAIN_GUI_ENTRY_BOOL,	"Allow sending Password unencrypted",
		NULL,	"cifsDialogLoginAllowUnencrypted", MAIN_GUI_PERM_ANYBODY, 
		FLG_0, ALLOW_NULL,	PANEL_OFFSET(allowUnencrypted)
	},
	{	MAIN_GUI_ENTRY_BOOL,	"Store Username and Password",
		NULL,	"cifsDialogLoginStore", MAIN_GUI_PERM_ANYBODY, 
		FLG_0, ALLOW_NULL,	PANEL_OFFSET(storePasswd)
	},
	{	MAIN_GUI_ENTRY_BOOL,	"Store Credentials as Default",
		NULL,	"cifsDialogLoginStoreDefault", MAIN_GUI_PERM_ANYBODY, 
		FLG_0, ALLOW_NULL,	PANEL_OFFSET(storeAsDefault)
	},
};

static guiEntry_t	sharePwPanelEntries[] = {
	{	MAIN_GUI_ENTRY_PASSWORD,	"Share Password",
		NULL,	"cifsDialogSharePwPassword", MAIN_GUI_PERM_ANYBODY, 
		MAIN_GUI_ENTRY_FLG_DEFAULT_RESPONDER,
		ALLOW_NULL,	PANEL_OFFSET(passwd)
	},
	{	MAIN_GUI_ENTRY_BOOL,	"Allow sending Password unencrypted",
		NULL,	"cifsDialogSharePwAllowUnencrypted", MAIN_GUI_PERM_ANYBODY, 
		FLG_0, ALLOW_NULL,	PANEL_OFFSET(allowUnencrypted)
	},
};


static guiButton_t	panelButtons[] = {
	{	"Cancel",			MAIN_GUI_METHOD_CANCEL,
		0,	/* parameter */	MAIN_GUI_BUTTON_CLOSE_DIALOG | MAIN_GUI_BUTTON_EQUALS_CLOSEBOX,
		"cifsDialogLoginButtonCancel",
	},
	{	"Revert",			MAIN_GUI_METHOD_GET,
		0,	/* parameter */	MAIN_GUI_BUTTON_ACTIVE_IF_DIRTY,	/* flags */
		"cifsDialogLoginButtonRevert",
	},
	{	"Login",			MAIN_GUI_METHOD_SET,
		0,	/* parameter */	MAIN_GUI_BUTTON_CLOSE_DIALOG | MAIN_GUI_BUTTON_DEFAULT,	/* flags */
		"cifsDialogLoginButtonLogin",
	},
};

/* ------------------------------------------------------------------------- */

static void	uirepGetPassPhraseProc(void *obj, void *arg)
{
uirepCtx_t		*this = obj;
btint			i;
guimanRes_t		*rval = arg;
ipcMsg_t		*msg = arg;
btu8			*p;
uirepRes_t		res;

	PROC_BEGIN;
	this->guiClass.type = MAIN_GUI_CLASS_PLAINCFG;
	this->guiClass.name = this->scratch;
	this->guiClass.entries = arrNew(NULL, 8);
	for(i=0;i<ELEMENTS(passPhraseEntries);i++)
		arrAddObject(this->guiClass.entries, &passPhraseEntries[i]);
	this->guiClass.buttons = arrNew(NULL, 4);
	for(i=0;i<ELEMENTS(panelButtons);i++)
		arrAddObject(this->guiClass.buttons, &panelButtons[i]);
	this->clients = ipcClientServicesForUid(mainModule->ipcModule, this->uid);
	memset(&res, 0, sizeof(res));	/* result if no client found */
	for(this->index=0;this->index<arrObjectCount(this->clients);this->index++){
		this->client = arrObjectAtIndex(this->clients, this->index);
		if(ipcSrvClientIsInvalid(this->client))
			continue;
		/* first try the specific method for the commandline interface */
		msg = ipcMsgNew();
		ipcAddInt(msg, 0);	/* size */
		ipcAddInt(msg, 0);	/* protocol version */
		ipcAddInt(msg, 0);	/* sequence number */
		ipcAddInt(msg, MAIN_IPCCFN_ASK_DATA);
		ipcAddInt(msg, 0);	/* uid */
		ipcAddInt(msg, 0);	/* gid */
		ipcAddAscii(msg, this->scratch);	/* prompt */
		ipcAddInt(msg, 0);	/* without echo */
		PROC_WAITFOR_X(1, ipcSrvRPC(mainModule->ipcModule, this->client, msg, this));
		memset(&res, 0, sizeof(res));
		if(msg == NULL){	/* client died */
			res.err.class = ERRCLS_FRAMEWORK;
			res.err.code = ERR_FRWclientcomm;
			goto doReturn;
		}
		ipcGetInt(msg);	/* uid */
		ipcGetInt(msg);	/* gid */
		res.err.class = ipcGetInt(msg);
		res.err.code = ipcGetInt(msg);
		if(res.err.class == 0){
			p = ipcGetAscii(msg);	/* pass phrase */
			if(p == NULL)
				goto doReturn;	/* cancel */
			res.isOk = 1;
			if(this->auth->passwd != NULL)
				memFree(this->auth->passwd);
			this->auth->passwd = utilStrDup(p);
			goto doReturn;
		}
		/* if cmdline interface failed, try generic GUI method */
		PROC_WAITFOR(guimanRunDialog(&this->guiClass, this->client, this->auth, 1, 1, this));
		if(rval->err.class == ERRCLS_FRAMEWORK){
			if(rval->err.code == ERR_FRWipcnotimpl)
				continue;	/* try next client */
		}
		memset(&res, 0, sizeof(res));
		if(rval->err.class != 0){
			res.err = rval->err;
			goto doReturn;
		}else{
			res.err.class = res.err.code = 0;
			res.isOk = rval->didSet;
			goto doReturn;
		}
	}
doReturn:
	arrFree(this->guiClass.entries);
	arrFree(this->guiClass.buttons);
	res.client = this->client;
	PROC_RETURN_AND_CLEANUP(&res,
		arrFree(this->clients);
	);
	PROC_END;
}

void	uirepGetPassPhrase(btint uid, btu8 *srvName, uirepAuthData_t *auth, void *caller)
{
uirepCtx_t	*ctx;

	if(uirepPool == NULL)
		uirepPool = poolNew(sizeof(uirepCtx_t), sizeof(uirepCtx_t));
	PROC_INIT(ctx, uirepGetPassPhraseProc, caller, uirepPool, poolFreeBlock);
	ctx->uid = uid;
	ctx->auth = auth;
	if(strlen(srvName) > ELEMENTS(ctx->scratch) - 20)
		srvName[ELEMENTS(ctx->scratch) - 20] = 0;	/* emergency brake */
	sprintf(ctx->scratch, "Pass Phrase for Server %s: ", srvName);
	PROC_START(ctx, NULL);
}

/* ------------------------------------------------------------------------- */

static void	uirepGetCredentialsProc(void *obj, void *arg)
{
uirepCtx_t		*this = obj;
btint			i;
guimanRes_t		*rval = arg;
btu8			*name, *passwd, *domain;
ipcMsg_t		*msg = arg;
uirepRes_t		res;

	PROC_BEGIN;
	this->guiClass.type = MAIN_GUI_CLASS_PLAINCFG;
	this->guiClass.name = this->scratch;
	this->guiClass.entries = arrNew(NULL, 8);
	for(i=0;i<ELEMENTS(loginPanelEntries);i++)
		arrAddObject(this->guiClass.entries, &loginPanelEntries[i]);
	this->guiClass.buttons = arrNew(NULL, 4);
	for(i=0;i<ELEMENTS(panelButtons);i++)
		arrAddObject(this->guiClass.buttons, &panelButtons[i]);
	this->clients = ipcClientServicesForUid(mainModule->ipcModule, this->uid);
	memset(&res, 0, sizeof(res));	/* result if no client found */
	for(this->index=0;this->index<arrObjectCount(this->clients);this->index++){
		this->client = arrObjectAtIndex(this->clients, this->index);
		if(ipcSrvClientIsInvalid(this->client))
			continue;
		/* first try the specific method for the commandline interface */
		msg = ipcMsgNew();
		ipcAddInt(msg, 0);	/* size */
		ipcAddInt(msg, 0);	/* protocol version */
		ipcAddInt(msg, 0);	/* sequence number */
		ipcAddInt(msg, MAIN_IPCCFN_ASK_PASSWORD);
		ipcAddInt(msg, 0);	/* uid */
		ipcAddInt(msg, 0);	/* gid */
		ipcAddAscii(msg, this->host);	/* hostname */
		PROC_WAITFOR(ipcSrvRPC(mainModule->ipcModule, this->client, msg, this));
		memset(&res, 0, sizeof(res));
		if(msg == NULL){	/* client died */
			res.err.class = ERRCLS_FRAMEWORK;
			res.err.code = ERR_FRWclientcomm;
			goto doReturn;
		}
		ipcGetInt(msg);	/* uid */
		ipcGetInt(msg);	/* gid */
		res.err.class = ipcGetInt(msg);
		res.err.code = ipcGetInt(msg);
		if(res.err.class == 0){
			name = ipcGetAscii(msg);	/* username */
			passwd = ipcGetAscii(msg);	/* username */
			domain = ipcGetAscii(msg);	/* username */
			if(name == NULL || passwd == NULL)
				goto doReturn;	/* cancel */
			if(this->auth->username != NULL)
				memFree(this->auth->username);
			this->auth->username = utilStrDup(name);
			if(this->auth->passwd != NULL)
				memFree(this->auth->username);
			this->auth->passwd = utilStrDup(passwd);
			if(domain != NULL && *domain != 0){
				if(this->auth->domain != NULL)
					memFree(this->auth->domain);
				this->auth->domain = utilStrDup(domain);
			}
			this->auth->allowUnencrypted = ipcGetInt(msg);
			res.isOk = 1;
			goto doReturn;
		}
		if(res.err.class == ERRCLS_FRAMEWORK && res.err.code == ERR_FRWusrabort)
			goto doReturn;
		/* if cmdline interface failed, try generic GUI method */
		if(this->auth->passwd != NULL){
			memFree(this->auth->passwd);
			this->auth->passwd = NULL;
		}
		PROC_WAITFOR(guimanRunDialog(&this->guiClass, this->client, this->auth, 1, 1, this));
		if(rval->err.class == ERRCLS_FRAMEWORK && rval->err.code == ERR_FRWipcnotimpl)
			continue;	/* try next client */
		memset(&res, 0, sizeof(res));
		res.err = rval->err;
		if(rval->err.class == 0)
			res.isOk = rval->didSet;
		goto doReturn;
	}
	/* if the loop runs through: implicit result = OK, user cancelled */
doReturn:
	arrFree(this->guiClass.entries);
	arrFree(this->guiClass.buttons);
	res.client = this->client;
	PROC_RETURN_AND_CLEANUP(&res,
		arrFree(this->clients);
	);
	PROC_END;
}

void	uirepGetCredentials(btint uid, btu8 *host, uirepAuthData_t *auth, void *caller)
{
uirepCtx_t	*ctx;

	if(uirepPool == NULL)
		uirepPool = poolNew(sizeof(uirepCtx_t), sizeof(uirepCtx_t));
	PROC_INIT(ctx, uirepGetCredentialsProc, caller, uirepPool, poolFreeBlock);
	ctx->uid = uid;
	ctx->auth = auth;
	if(strlen(host) > ELEMENTS(ctx->scratch) - 20)
		host[ELEMENTS(ctx->scratch) - 20] = 0;	/* emergency brake */
	sprintf(ctx->scratch, "Login on Host %s", host);
	strcpy(ctx->host, host);
	PROC_START(ctx, NULL);
}

/* ------------------------------------------------------------------------- */

static void	uirepGetSharePasswordProc(void *obj, void *arg)
{
uirepCtx_t		*this = obj;
btint			i;
guimanRes_t		*rval = arg;
btu8			*p;
ipcMsg_t		*msg = arg;
uirepRes_t		res;

	PROC_BEGIN;
	this->guiClass.type = MAIN_GUI_CLASS_PLAINCFG;
	this->guiClass.name = this->scratch;
	this->guiClass.entries = arrNew(NULL, 8);
	for(i=0;i<ELEMENTS(sharePwPanelEntries);i++)
		arrAddObject(this->guiClass.entries, &sharePwPanelEntries[i]);
	this->guiClass.buttons = arrNew(NULL, 4);
	for(i=0;i<ELEMENTS(panelButtons);i++)
		arrAddObject(this->guiClass.buttons, &panelButtons[i]);
	this->clients = ipcClientServicesForUid(mainModule->ipcModule, this->uid);
	memset(&res, 0, sizeof(res));
	for(this->index=0;this->index<arrObjectCount(this->clients);this->index++){
		this->client = arrObjectAtIndex(this->clients, this->index);
		if(ipcSrvClientIsInvalid(this->client))
			continue;
		/* first try the specific method for the commandline interface */
		msg = ipcMsgNew();
		ipcAddInt(msg, 0);	/* size */
		ipcAddInt(msg, 0);	/* protocol version */
		ipcAddInt(msg, 0);	/* sequence number */
		ipcAddInt(msg, MAIN_IPCCFN_ASK_SHAREPASSWD);
		ipcAddInt(msg, 0);	/* uid */
		ipcAddInt(msg, 0);	/* gid */
		ipcAddAscii(msg, this->host);	/* share name */
		PROC_WAITFOR(ipcSrvRPC(mainModule->ipcModule, this->client, msg, this));
		memset(&res, 0, sizeof(res));
		if(msg == NULL){	/* client died */
			res.err.class = ERRCLS_FRAMEWORK;
			res.err.code = ERR_FRWclientcomm;
			goto doReturn;
		}
		ipcGetInt(msg);	/* uid */
		ipcGetInt(msg);	/* gid */
		res.err.class = ipcGetInt(msg);
		res.err.code = ipcGetInt(msg);
		if(res.err.class == 0){
			p = ipcGetAscii(msg);	/* passwd */
			if(p == NULL)
				goto doReturn;	/* cancel */
			if(this->auth->passwd != NULL)
				memFree(this->auth->passwd);
			this->auth->passwd = utilStrDup(p);
			this->auth->allowUnencrypted = ipcGetInt(msg);
			res.isOk = 1;
			goto doReturn;
		}
		if(res.err.class == ERRCLS_FRAMEWORK && res.err.code == ERR_FRWusrabort)
			goto doReturn;
		/* if cmdline interface failed, try generic GUI method */
		if(this->auth->passwd != NULL){
			memFree(this->auth->passwd);
			this->auth->passwd = NULL;
		}
		PROC_WAITFOR(guimanRunDialog(&this->guiClass, this->client, this->auth, 1, 1, this));
		if(rval->err.class == ERRCLS_FRAMEWORK && rval->err.code == ERR_FRWipcnotimpl)
			continue;	/* try next client */
		memset(&res, 0, sizeof(res));
		res.err = rval->err;
		if(rval->err.class == 0)
			res.isOk = rval->didSet;
		goto doReturn;
	}
	/* if the loop runs through: implicit result = OK, user cancelled */
doReturn:
	arrFree(this->guiClass.entries);
	arrFree(this->guiClass.buttons);
	res.client = this->client;
	PROC_RETURN_AND_CLEANUP(&res,
		arrFree(this->clients);
	);
	PROC_END;
}

void	uirepGetSharePassword(btint uid, btuni *share, uirepAuthData_t *auth, void *caller)
{
uirepCtx_t	*ctx;
btint		len;

	if(uirepPool == NULL)
		uirepPool = poolNew(sizeof(uirepCtx_t), sizeof(uirepCtx_t));
	PROC_INIT(ctx, uirepGetSharePasswordProc, caller, uirepPool, poolFreeBlock);
	ctx->uid = uid;
	ctx->auth = auth;
	convUniToLocalAscii(share, ctx->host, ELEMENTS(ctx->host));
	strcpy(ctx->scratch, "Connect to Share ");
	len = strlen(ctx->scratch);
	strncpy(ctx->scratch + len, ctx->host, sizeof(ctx->scratch) - 1 - len);
	ctx->scratch[ELEMENTS(ctx->scratch) - 1] = 0;
	PROC_START(ctx, NULL);
}

/* ------------------------------------------------------------------------- */
