/*
 * Name: help.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-04-02
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: help.h,v 1.1.1.1 1999/04/12 13:56:39 cs Exp $
 */

#ifndef __help_H_INCLUDED__
#define __help_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module implements parsing of help-files and the help database. It's
currently implemented as a simple hashtable of help items.
*/

/* ------------------------------------------------------------------------- */
void	helpParseFile(btu8 *helpFile);
/* This function parses a help-file and adds all items it finds to the internal
 * database. Help-files are a subset of HTML. The association between sections
 * in the help file and GUI objects is made through item names. Item names are
 * written in HTML comments in a line of their own in the help file.
 * Parameters:
 *    'helpFile' is the full path to a file containing the help texts.
 */
btu8	*helpTextForItem(btu8 *item);
/* This function looks up the help text for a particular item. If there's no
 * help for the item, NULL is returned (this is different from the case where
 * help is available in general, but no item has been defined in the file). If
 * no associated help item has been found in the database, the item itself is
 * returned. This allows the user to add an item to the help file if desired.
 * Parameters:
 *    'item' is the name associated to a GUI object.
 * Returns: The help text or NULL, if no help is available. If no help is
 *    defined (but should be), the same pointer as passed in 'item' is
 *    returned.
 */
/* ------------------------------------------------------------------------- */

#endif /* __help_H_INCLUDED__ */
