/*
 * Name: ipc.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-11-27
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: ipc.h,v 1.1.1.1 1999/04/12 13:56:38 cs Exp $
 */

#ifndef __ipc_H_INCLUDED__
#define __ipc_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module implements the basic functions used for RPC between user
interface commands and the daemon. The basic marshalling and unmarshalling
functions are declared here as well as the message datatype, the function
indices and the error codes.
*/

/* ------------------------------------------------------------------------- */

typedef struct ipcMsg{
	btu8	*buffer;
	btsize	allocated;
	btsize	size;
	btsize	index;
}ipcMsg_t;

/* ------------------------------------------------------------------------- */

ipcMsg_t	*ipcMsgNew(void);
void	ipcMsgFree(ipcMsg_t *this);
void	ipcMsgClear(ipcMsg_t *this);
void	ipcMsgEnsureSize(ipcMsg_t *this, btsize bytesNeeded);

void	ipcAddInt(ipcMsg_t *this, btu32 val);
void	ipcAddBlock(ipcMsg_t *this, btu8 *block, btsize len);
void	ipcAddAscii(ipcMsg_t *this, btu8 *asciiString);

btu32	ipcGetInt(ipcMsg_t *this);
btsize	ipcGetBlock(ipcMsg_t *this, btu8 **block);
btu8	*ipcGetAscii(ipcMsg_t *this);

/* function IDs for IPC calls (must match table in ipcsrvstubs.c): */
/* These are the IDs for calls from the commanline tools to the daemon: */
#define	IPC_FN_GET_VERSION			0
#define	IPC_FN_SLASH_CHARS			1
#define	IPC_FN_MOUNT_SHARE			2
#define	IPC_FN_MOUNT_BROWSER		3
#define	IPC_FN_LOGIN				4
#define	IPC_FN_UMOUNT				5
#define	IPC_FN_LOGOUT				6
#define	IPC_FN_SET_LICENSE			7
#define	IPC_FN_GET_LICENSE			8
#define	IPC_FN_IS_LICENSED			9
#define	IPC_FN_CURRENT_LLOAD		10
#define	IPC_FN_CURRENT_SERVERS		11
#define	IPC_FN_REGISTER_PWSRV		12
#define	IPC_FN_UNREGISTER_PWSRV		13
#define	IPC_FN_READ_CONFIG			14
#define	IPC_FN_WRITE_CONFIG			15
#define	IPC_FN_LIST_SERVERS			16
#define	IPC_FN_LIST_BROWSERS		17
#define	IPC_FN_SET_FAKEMODE			18
#define	IPC_FN_LIST_MOUNTS			19
#define	IPC_FN_LIST_LOGINS			20
#define	IPC_FN_STORE_MOUNT			21
#define	IPC_FN_UNSTORE_MOUNT		22
#define	IPC_FN_STORE_LOGIN			23
#define	IPC_FN_UNSTORE_LOGIN		24
#define	IPC_FN_DELETE_CONFIG		25
#define	IPC_FN_HAS_SSL_OPTION		26
#define	IPC_FUNCTION_COUNT			27

/* These are the IDs for calls from the daemon to the client: */
#define	IPC_CFN_GETPASS				0
#define	IPC_CFN_GETSSLPHRASE		1
#define	IPC_CFN_NOTIFY_UPDATE		2
#define	IPC_CFN_FUNCTION_COUNT		3



/* ------------------------------------------------------------------------- */

/* constants for mount status: */
#define	IPC_MSTAT_NOTMOUNTED	0	/* not mounted */
#define	IPC_MSTAT_MOUNT			1	/* manually mounted */
#define	IPC_MSTAT_AUTOMOUNT		2	/* mounted by browser-automount */

/* constants for dynamic typing: */
#define	IPC_TYPE_STRING	0	/* encoded with ipcAddAscii() */
#define	IPC_TYPE_ENUM	1	/* encoded with ipcAddInt() */
#define	IPC_TYPE_INT	2	/* encoded with ipcAddInt() */
#define	IPC_TYPE_BOOL	3	/* encoded with ipcAddInt() */
#define	IPC_TYPE_NONE	4	/* no data, no type */

/* constants for configuration context: */
#define	IPC_CFGCTX_GLOBAL	0
#define	IPC_CFGCTX_LICENSE	1
#define	IPC_CFGCTX_SERVER	2
#define	IPC_CFGCTX_BROWSER	3

/* constants for update notification: */
#define	IPC_NOTIFY_UPDATE_CFG		0	/* config or stored DB changed */
#define	IPC_NOTIFY_UPDATE_MOUNTS	1	/* mount list changed */
#define	IPC_NOTIFY_UPDATE_LOGINS	2	/* login list changed */
#define	IPC_NOTIFY_UPDATE_ALL		3	/* anything changed */
#define	IPC_NOTIFY_UPDATE_TYPES		4	/* number of types defined */

/* ------------------------------------------------------------------------- */

#endif	/* __ipc_H_INCLUDED__ */
