/*
 * Name: memalloc.c
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-12
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: memalloc.c,v 1.1.1.1 1999/04/12 13:56:38 cs Exp $
 */

#include "stdheaders.h"
#include "memalloc.h"
#include "logging.h"

/* ------------------------------------------------------------------------- */

EXC_DEFINE(memExcNomem);

/* ------------------------------------------------------------------------- */

void	*memAlloc(btsize size)
{
void	*p;

	if(size <= 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_RESOURCE))
			logPrintf(LOGM_RESOURCE, "allocating %d bytes\n", size);
		return NULL;
	}
	if((p = malloc(size)) == NULL){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "error allocating %d bytes of memory\n", size);
		EXC_THROW(memExcNomem);
	}
	return p;
}

/* ------------------------------------------------------------------------- */

void	*memCalloc(btsize size)
{
void	*p;

	if(size <= 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_RESOURCE))
			logPrintf(LOGM_RESOURCE, "callocating %d bytes\n", size);
		return NULL;
	}
	if((p = calloc(size, 1)) == NULL){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "error allocating %d bytes of memory\n", size);
		EXC_THROW(memExcNomem);
	}
	return p;
}

/* ------------------------------------------------------------------------- */

void	*memRealloc(void *oldbase, btsize newsize)
{
void	*p;

	if(newsize <= 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_RESOURCE))
			logPrintf(LOGM_RESOURCE, "reallocating to %d bytes\n", newsize);
		memFree(oldbase);
		return NULL;
	}
    if(oldbase == NULL){
        p = malloc(newsize);
    }else{
        p = realloc(oldbase, newsize);
    }
	if(p == NULL){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "error reallocating to %d bytes\n", newsize);
		EXC_THROW(memExcNomem);
	}
	return p;
}

/* ------------------------------------------------------------------------- */

void	memFree(void *base)
{
	if(base != NULL){
		free(base);
	}
}

/* ------------------------------------------------------------------------- */

