/*
 * Name: scanner.h
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1996-10-04
 * Tabsize: 4
 * Copyright: (c) 1996, 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: scanner.h,v 1.1.1.1 1999/04/12 13:56:39 cs Exp $
 */

#ifndef __scanner_H_INCLUDED__
#define __scanner_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module implements the lexical scanner used by the configuration file
parser in "cfgparse.c". It's a general purpose scanner.
*/

/* -------------------------- instance variables: -------------------------- */

typedef struct scanner{
	int		(*getc_prog)(void *arg);/* getc() function					*/
	void	*arg;				/* argument for getc()					*/
	int		ungetc_val;			/* storage for backtracking				*/
	int		unget_token;		/* storage to push back token			*/
	btu8	*token_buffer;		/* buffer for token values				*/
	int		buffer_len;			/* length of allocated buffer			*/
	int		buffer_i;			/* index into buffer					*/
	int		line;				/* actual line number					*/
	btbool	ungetc_valid;		/* backtracking store contains data		*/
	btbool	unget_valid;		/* token was pushed back				*/
	btbool	allow_crem;			/* C-style remarks are allowed			*/
	btbool	allow_cpprem;		/* C++ style remarks (//) are allowed	*/
	btbool	allow_shrem;		/* shell style remarks (#) are allowed	*/
	btbool	nl_is_token;		/* true if '\n' is a token				*/
}scanner_t;

/* ------------------------------- constants ------------------------------- */

#define	SCAN_TOKEN_EOF			-1	/* end of stream reched					*/
#define	SCAN_TOKEN_ALNUM		1	/* scanned token is alphanumeric		*/
#define	SCAN_TOKEN_STRING		2	/* scanned token is a string			*/
#define	SCAN_TOKEN_CHAR			3	/* scanned token is a character const	*/
#define	SCAN_TOKEN_OPERATOR		4	/* scanned token is something else		*/


/* ------------------------------ prototypes ------------------------------- */

scanner_t	*scannerNew(int (*getc_prog)(), void *arg);
void	scannerFree(scanner_t *this);
void	scannerSetAllowRem(scanner_t *this, btbool crem, btbool cpprem, btbool shrem);
void	scannerSetNlIsToken(scanner_t *this, btbool is_token);
void	scannerSetGetc(scanner_t *this, int (*getc_prog)(void *arg), void *arg);
int		scannerGetLineNumber(scanner_t *this);
int		scannerGetToken(scanner_t *this, btu8 **value);
void	scannerUngetToken(scanner_t *this, int token);

#endif /* __scanner_H_INCLUDED__ */
