/*
 * Name: module.c
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-02-10
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: module.c,v 1.3 2001/11/06 20:02:54 cs Exp $
 */

#include "stdheaders.h"
#include "module.h"
#include "array.h"
#include "hashtable.h"
#include "notification.h"
#include "util.h"

/* ------------------------------------------------------------------------- */

static hashtable_t			*allModules;
static notificationCenter_t	*registerNotification;

/* The following are for sorted enumerations: */
static array_t	*allFrontends = NULL;
static array_t	*allBackends = NULL;

/* ------------------------------------------------------------------------- */

static btint	moduleCompare(void *o1, void *o2)
{
module_t	*m1 = o1, *m2 = o2;

	return strcmp(m1->name, m2->name);
}

static btu32	moduleHashVal(void *obj)
{
module_t	*this = obj;

	return utilStrHash(this->name, 1);
}

static collectionObjectI_t	moduleInterface = {
	moduleCompare,
	moduleHashVal,
	NULL,
	NULL
};

/* ------------------------------------------------------------------------- */

void	modRegister(module_t *mod)
{
	if(allModules == NULL){
		allModules = htabNew(&moduleInterface, 8, 75);
	}
	htabAddObject(allModules, mod);
	if(mod->moduleType == MOD_TYPE_FRONTEND || mod->moduleType == MOD_TYPE_BACKEND){
		if(allFrontends != NULL)
			arrRemoveAllObjects(allFrontends);
		if(allBackends != NULL)
			arrRemoveAllObjects(allBackends);
	}
	if(registerNotification != NULL){
		ncPostNotification(registerNotification, mod);
	}
}

/* ------------------------------------------------------------------------- */

struct notificationCenter	*modRegisterNotification(void)
{
	if(registerNotification == NULL){
		registerNotification = ncNew(4);
	}
	return registerNotification;
}

/* ------------------------------------------------------------------------- */

struct array	*modAllModules(void)
{
	return htabAllObjects(allModules);
}

/* ------------------------------------------------------------------------- */

module_t	*modFind(btu8 *name)
{
	return htabFindObjectEqualTo(allModules, &name);
}

/* ------------------------------------------------------------------------- */

static btint	modPrecedenceCompare(void *o1, void *o2)
{
module_t	*this = o1, *other = o2;

	return other->sortPrecedence - this->sortPrecedence;
}

static collectionObjectI_t	modPrecedenceInterface = {
	modPrecedenceCompare,
	NULL,
	NULL,
	NULL
};

static void	updateAllArrays(void)
{
array_t		*arr;
btint		i, cnt;
module_t	*mod;

	if(allFrontends == NULL)
		allFrontends = arrNew(&modPrecedenceInterface, 4);
	if(allBackends == NULL)
		allBackends = arrNew(&modPrecedenceInterface, 4);
	/* if invalidated because new modules added: */
	if(arrObjectCount(allFrontends) == 0 && arrObjectCount(allBackends) == 0){
		arr = modAllModules();
		cnt = arrObjectCount(arr);
		for(i=0;i<cnt;i++){
			mod = arrObjectAtIndex(arr, i);
			if(mod->moduleType == MOD_TYPE_FRONTEND)
				arrAddObject(allFrontends, mod);
			else if(mod->moduleType == MOD_TYPE_BACKEND)
				arrAddObject(allBackends, mod);
		}
		arrFree(arr);
		arrSort(allFrontends);
		arrSort(allBackends);
	}
}

struct array	*modAllFrontends(void)
{
	updateAllArrays();
	return allFrontends;
}

struct array	*modAllBackends(void)
{
	updateAllArrays();
	return allBackends;
}

/* ------------------------------------------------------------------------- */
