/*
 * Name: module.h
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-02-10
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: module.h,v 1.3 2001/11/06 09:50:36 cs Exp $
 */

#ifndef __module_H_INCLUDED__
#define __module_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module implements the administration of modules. Modules can be either
compiled in or (if there's an implementation for dynamic loading on the
platform) loaded dynamically.
*/

/* ------------------------------------------------------------------------- */

typedef struct module{
	btu8				*name;			/* name of this module */
	btint				moduleType;		/* see the defines below */
	void				*(*implementationOf)(btu8 *functionName);	/* access to interface functions */
	struct ipcModule	*ipcModule;		/* how this module communicates with the user interface */
	btu32				interfaceVersion;		/* version of the module's function interface */
	btu32				implementationVersion;	/* version of the module's implementation */
	btu32				betaVersion;			/* if != 0: beta version counter */
	btint				sortPrecedence;			/* where to show in popups, etc. Higher is better */
	btint				refCount;				/* not currently used */
	btbool				wasDynamicallyLoaded;	/* not currently used */
}module_t;

/* valid module types: */
#define	MOD_TYPE_FRAMEWORK	0
#define	MOD_TYPE_FRONTEND	1
#define	MOD_TYPE_BACKEND	2
#define	MOD_TYPE_OTHER		3

/* ------------------------------------------------------------------------- */

void	modRegister(module_t *mod);
/* This function registers a new module with the framework. The main purpose
 * of registering modules is to allow other modules to look it up by name.
 * Parameters: 'mod': an allocated and fully filled module_t structure.
 */
struct notificationCenter	*modRegisterNotification(void);
/* This function returns a notification center that fires when a new module
 * is registered. The new module is parameter of the notification. This allows
 * modules to watch out for new modules without having to poll.
 * Returns: non-retained notification center object.
 */
struct array	*modAllModules(void);
/* This function returns an array of all currently registered modules. The
 * caller is responsible for freeing the array.
 */
module_t	*modFind(btu8 *name);
/* This function looks up a module by name. If the module is not found, NULL
 * is returned.
 */
struct array	*modAllFrontends(void);
/* This function returns an ordered array of frontend modules.
 * The array is static and valid during the lifetime of the daemon. Callers
 * MUST NOT free it.
 */
struct array	*modAllBackends(void);
/* This function returns an ordered array of backend modules.
 * The array is static and valid during the lifetime of the daemon. Callers
 * MUST NOT free it.
 */

#endif /* __module_H_INCLUDED__ */
